// version 2

#ifndef MMAPFILE_HPP
#define MMAPFILE_HPP

#include "Platform.hpp"

#if defined(_WIN32)
# define WIN32_LEAN_AND_MEAN
# include <windows.h>
#endif

class MMapFile
{
	char *data;
	u32 len;
	s32 offset;

#if defined(__linux__)
	int fd;
#elif defined(_WIN32)
	HANDLE hFile, hMapping;
#endif

public:
	MMapFile(const char *path);
	~MMapFile();

	inline bool good() { return data != 0; }
	inline bool inside() { return offset >= 0 && offset < (s32)len; }

	inline u32 size() { return len; }

	inline void seek(s32 poffset) { offset = poffset; }
	inline bool underrun(s32 requested) { return (u32)(offset + requested) > len; }
	inline char *look() { return data + offset; }
	inline char *look(s32 offset) { return data + offset; }
	inline char *read(s32 requested) { offset += requested; return data + (offset - requested); }
	inline void skip(s32 requested) { offset += requested; }
	inline u32 remaining() { return len - offset; }
	inline u32 getOffset() { return offset; }
};


#endif // include guard
